@@ -1,11 +1,9 @@
 from __future__ import annotations
 
-import os
 from io import BytesIO
-from pathlib import Path
 
-from pyisotools.iohelper import (read_string, read_ubyte, read_uint16, read_uint32,
-                                 write_ubyte, write_uint16, write_uint32)
+from pyisotools.iohelper import (read_string, read_ubyte, read_uint32,
+                                 write_ubyte, write_uint32)
 
 
 class Boot(object):
